/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.realworld.manager;

import java.util.Date;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.core.Events;
import org.richfaces.realworld.domain.Comment;
import org.richfaces.realworld.domain.Image;
import org.richfaces.realworld.domain.User;
import org.richfaces.realworld.service.Constants;
import org.richfaces.realworld.service.IImageAction;
import org.richfaces.realworld.util.DirectLinkHelper;
import org.richfaces.realworld.util.FlushStrategy;


@Name("imageManager")
@Scope(ScopeType.CONVERSATION)
public class ImageManager {

	@In(create=true, required=true)
	private IImageAction imageAction;
	@In FlushStrategy flushStrategy;
	@In(create=true)
	FileManager fileManager;
	
	@In @Out Model model;
	
	@In
	private User user;
	
	@In(create=true)
	private DirectLinkHelper directLink;
	
	public void deleteImage(Image image){
		String imagePath = image.getPath();
		imageAction.deleteImage(image, flushStrategy.isDatabaseStoreStrategy());
		fileManager.deleteImage(imagePath);
		Events.instance().raiseEvent("imageDeleted");
	}

	public void editImage(Image image){
		imageAction.editImage(image, flushStrategy.isDatabaseStoreStrategy());
		Events.instance().raiseEvent(Constants.UPDATE_MAIN_AREA_EVENT, NavigationEnum.ALBUM_IMAGE_PREVIEW);
		Events.instance().raiseEvent("imageEdited");
	}
	
	@Observer(Constants.ADD_IMAGE_EVENT)
	public void addImage(Image image) {
		imageAction.addImage(image, flushStrategy.isDatabaseStoreStrategy());
		model.setSelectedAlbum(image.getAlbum());
		Events.instance().raiseEvent(Constants.UPDATE_MAIN_AREA_EVENT, NavigationEnum.ALBUM_IMAGE_PREVIEW);
		Events.instance().raiseEvent("imageAdded");
	}

	public void addComment(Image image, String message){
		if(null  == user.getLogin()){
			Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, new Exception("For vote on this picture you must login"));
			return;
		}
		Comment comment= new Comment();
		comment.setAuthor(user);
		comment.setImage(image);
		comment.setDate(new Date());
		comment.setMessage(message);
		imageAction.addComment(comment, flushStrategy.isDatabaseStoreStrategy());
		Events.instance().raiseEvent(Constants.CLEAR_EDITOR_EVENT, "");
	}
	
	public void deleteComment(Comment comment){
		imageAction.deleteComment(comment, flushStrategy.isDatabaseStoreStrategy());
	}

	public String getImageDirectLink(Image image) {
		return directLink.getDirectImageLink(image.getId());
	}

}
